% GOAT_AutoDownload_ERSST
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_ERSST(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'NCDC ERSST V3b' ;
        D.AutoDownloadDataset.FolderName = 'ERSST' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['The NOAA Extended Reconstruction Sea Surface Temperature (ERSST) provides global, spatially complete SST data at a monthly timestep for 1854-present.', ... 
                 '\nIt is based upon statistical interpolation of the ICOADS release 2.4 data.', ...
                 '\nVersion 3 is the latest release and includes satellite AVHRR SST data for 1985 onwards.', ... 
                 '\nVersion 3b does not include satellite data due to a cold bias in the satellite-derived SSTs that proved difficult to correct.', ...
                 '\nThe data are distributed as anomalies; climatologies (absolute values) are available for 1971-2000.', ...
                 '\nERSST v3 forms the basis for NOAA''s merged global land-ocean surface temperature analysis.', ...
                 '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ; 
        D.AutoDownloadDataset.CurrentYear = 1 ; 
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'tos'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1854 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Monthly' ;
        switch Field
            case 'tos'
                D.AutoDownloadDataset.FieldDescription = 'Sea surface temperature' ;
                D.AutoDownloadDataset.Units = 'C' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.ncdc.noaa.gov/pub/data/cmb/ersst/v3b/netcdf//ersst.yyyymm.nc' ;
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        for y = Year1:Year2
            for m = 1:12
                URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
                URL = regexprep(URL, 'mm', num2str(m,'%02.0f')) ;
                set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),' . . .']) ; drawnow ;
                FileName = ['ERSSTdata.nc'] ;
                for da = 1:10
                    try
                        urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                        break ;
                    catch
                        if da < 10
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                            pause(300) ;
                        else
                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),' Failed. Download Terminated.']) ; drawnow ;
                            return ;
                        end
                    end
                end
                Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
                try
                    load(fullfile(GOATAddress,'Dims.mat'))
                catch
                    lon = Data.lon ;
                    lat = Data.lat ;
                    P = 1000 ;
                    Units = D.Field.Units ;
                    VerticalOrdinate = D.Field.VerticalOrdinate ;
                    try
                        FieldDescription = D.Field.FieldDescription ;
                        save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        FieldDescription = [D.Field.FieldDescription,' Anomaly'] ;
                        save(fullfile([GOATAddress,'anom'],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        FieldDescription = [D.Field.FieldDescription,' Error'] ;
                        save(fullfile([GOATAddress,'err'],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    catch
                        mkdir([GOATAddress,'anom']) ;
                        mkdir([GOATAddress,'err']) ;
                        mkdir(GOATAddress) ;
                        FieldDescription = D.Field.FieldDescription ;
                        save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        FieldDescription = [D.Field.FieldDescription,' Anomaly'] ;
                        save(fullfile([GOATAddress,'anom'],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        FieldDescription = [D.Field.FieldDescription,' Error'] ;
                        save(fullfile([GOATAddress,'err'],'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    end
                end
                
                data = Data.var1(:,:) ;
                data(data<-9) = nan ;
                FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000),'.mat']);
                try
                    save(FileAddress,'data') ;
                catch
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                data = Data.var2(:,:) ;
                data(data<-9) = nan ;
                FileAddress = fullfile([GOATAddress,'anom'],int2str(y),[int2str(m*10000),'.mat']);
                try
                    save(FileAddress,'data') ;
                catch
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                data = Data.var3(:,:) ;
                data(data<-9) = nan ;
                FileAddress = fullfile([GOATAddress,'err'],int2str(y),[int2str(m*10000),'.mat']);
                try
                    save(FileAddress,'data') ;
                catch
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                delete(fullfile(DownloadFolder,FileName)) ;
            end
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
        
end

return