% GOAT_AutoDownload_GISTEMP
% Copyright (C) Ori Adam Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_GISTEMP(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'NASA-GISS GISTEMP250 Monthly' ;
        D.AutoDownloadDataset.FolderName = 'GISTEMP' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['NASA Goddard''s Global Surface Temperature Analysis (GISTEMP) combines land surface air temperatures', ...
            '\nprimarily from the GHCN-M version 3 with SSTs of the ERSSTv3b analysis into a comprehensive global surface temperature data set', ...
            '\nspanning 1880 to the present at monthly resolution, on a 2x2 degree latitude-longitude grid.', ...
            '\nAs such, it is one of the main data sets used to monitor global and regional temperature variability and trends.', ...
            '\nGISTEMP has greater polar coverage than MLOST or HadCRUT4, in part due to the inclusion of Antarctic ''READER'' stations and in part due to the interpolation method.', ...
            '\nLike NOAA''s MLOST, interpolation of the station records is performed to provide broad spatial coverage, however, the methodology is unique.', ...
            '\nThe GISTEMP analysis method uses linear inverse distance weighting to infill gridboxes with records from stations up to 1200km away (a version with a 250km limit is also provided).  ', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 0 ;  %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 0 ;
        D.AutoDownloadDataset.Fields.Names = {'ts_land','ts'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K)
        D.AutoDownloadDataset.FirstYear = 1880 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Monthly' ;
        switch Field
            case 'ts_land'
                D.AutoDownloadDataset.FieldDescription = 'GISTEMP land temperature anomaly (relative to 1951-1980). 250km smoothing.' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://data.giss.nasa.gov/pub/gistemp/gistemp250.nc.gz' ;
            case 'ts'
                D.AutoDownloadDataset.FieldDescription = 'GISTEMP land_ERSST OCean temperature anomaly (relative to 1951-1980). 1200km smoothing.' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://data.giss.nasa.gov/pub/gistemp/gistemp1200_ERSST.nc.gz' ;
        end
    case 'Download'
        FieldName = D.Field.Name ;
        year0 = 1880 ;
        DataFolder = D.DataSet.Address ;
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        set(D.h_message,'string',['Downloading ',D.Field.Name,' . . .']) ; drawnow;
        switch FieldName
            case 'ts_land'
                gunzip(D.Field.url,DownloadFolder) ;
                set(D.h_message,'string',['Installing ',D.Field.Name,' . . .']) ; drawnow;
                Data = GOAT_nc2mat(fullfile(DownloadFolder,'gistemp250.nc')) ;
                delete(fullfile(DownloadFolder,'gistemp250.nc')) ;
                mkdir(DataFolder) ;
                mkdir(fullfile(DataFolder,'Monthly')) ;
                mkdir(fullfile(DataFolder,'Monthly',FieldName)) ;
                P = 1000 ;
                lon = Data.lon ;
                lat = Data.lat ;
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                save(fullfile(DataFolder,'Monthly',FieldName,'Dims.mat'),'lat','lon','P','Units','VerticalOrdinate','FieldDescription') ;
                q = 0 ;
                for y = year0:year0 + floor(size(Data.var1,3)/12)
                    mkdir(fullfile(DataFolder,'Monthly',FieldName,int2str(y))) ;
                    for m = 1:12
                        q = q+1 ;
                        try
                            data = squeeze(Data.var1(:,:,q)) ;
                            data(data > 300) = nan ;
                            save(fullfile(DataFolder,'Monthly',FieldName,int2str(y),[int2str(m*10000),'.mat']),'data') ;
                        catch
                            %  nevermind, missing endyear files
                        end
                    end
                end
                
            case 'ts'
                gunzip(D.Field.url,DownloadFolder) ;
                set(D.h_message,'string',['Installing ',D.Field.Name,' . . .']) ; drawnow;
                Data = GOAT_nc2mat(fullfile(DownloadFolder,'gistemp1200_ERSST.nc')) ;
                delete(fullfile(DownloadFolder,'gistemp1200_ERSST.nc')) ;
                mkdir(fullfile(DataFolder,'Monthly',FieldName)) ;
                %
                P = 1000 ;
                lon = Data.lon ;
                lat = Data.lat ;
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                save(fullfile(DataFolder,'Monthly',FieldName,'Dims.mat'),'lat','lon','P','Units','VerticalOrdinate','FieldDescription') ;
                q = 0 ;
                for y = year0:year0 + floor(size(Data.var1,3)/12)
                    mkdir(fullfile(DataFolder,'Monthly',FieldName,int2str(y))) ;
                    for m = 1:12
                        q = q+1 ;
                        try
                            data = squeeze(Data.var1(:,:,q)) ;
                            data(data > 300) = nan ;
                            save(fullfile(DataFolder,'Monthly',FieldName,int2str(y),[int2str(m*10000),'.mat']),'data') ;
                        catch
                            %  nevermind, missing endyear files
                        end
                    end
                end
        end
        
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end
return 