% GOAT_AutoDownload_ORAS4
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_ORAS4(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'ECMWF ORAS4' ;
        D.AutoDownloadDataset.FolderName = 'ORAS4' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['ECMWF''s current ocean reanalysis uses a sophisticated data assimilation methodology which includes a model bias correction.', ...
            '\nThe ocean model used is forced by atmospheric daily surface fluxes, relaxed to SST and bias corrected.', ...
            '\nKey strengths:', ...
            '\nImproved high quality data assimilation method and model', ...
            '\nImproved surface forcing and quality control in-situ observations (XBT corrected, Argo blacklists...)', ...
            '\nKey limitations:', ...
            '\nFirst 2 decades should be used with caution (large uncertainties)', ...
            '\nAtlantic Meridional Circulation (AMOC) at 26N is underestimated', ...
            '\nLarge surface salinity errors.', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 0 ; %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'so','thetao','uo','vo','zos'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Depth (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1958 ;
        D.AutoDownloadDataset.LastYear = 2009 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Monthly' ;
        switch Field
            case 'so'
                D.AutoDownloadDataset.FieldDescription = 'Sea water salinity' ;
                D.AutoDownloadDataset.Units = 'Unitless' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.icdc.zmaw.de/EASYInit/ORA-S4/monthly_1x1/so_oras4_1m_YYYY_grid_1x1.nc.gz' ;
            case 'thetao'
                D.AutoDownloadDataset.FieldDescription = 'Sea water potential temperature' ;
                D.AutoDownloadDataset.Units = 'C' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.icdc.zmaw.de/EASYInit/ORA-S4/monthly_1x1/thetao_oras4_1m_YYYY_grid_1x1.nc.gz' ;
            case 'uo'
                D.AutoDownloadDataset.FieldDescription = 'Sea water zonal velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.icdc.zmaw.de/EASYInit/ORA-S4/monthly_1x1/uo_oras4_1m_YYYY_grid_1x1.nc.gz' ;
            case 'vo'
                D.AutoDownloadDataset.FieldDescription = 'Sea water meridional velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.icdc.zmaw.de/EASYInit/ORA-S4/monthly_1x1/vo_oras4_1m_YYYY_grid_1x1.nc.gz' ;
            case 'zos'
                D.AutoDownloadDataset.FieldDescription = 'Sea water height above geoid' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.icdc.zmaw.de/EASYInit/ORA-S4/monthly_1x1/zos_oras4_1m_YYYY_grid_1x1.nc.gz' ;
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        for y = Year1:Year2
            URL = regexprep(D.Field.url, 'YYYY', int2str(y)) ;
            set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
            %FileName = [int2str(y),D.Field.Name,'ORAS.gz'] ;
            [FilePath, FileName] = fileparts(URL) ;
            for da = 1:10
                try
                    gunzip(URL,DownloadFolder) ;
                    break ;
                catch
                    if da < 10
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                        pause(300) ;
                    else
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                        return ;
                    end
                end
            end
            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
            GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
            try
                load(fullfile(GOATAddress,'Dims.mat'))
            catch
                lon = Data.lon ;
                lat = Data.lat ;
                switch D.Field.LevelType
                    case '3D'
                        P = Data.P ;
                    case 'Surface'
                        P = 0 ;
                end
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                try
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                catch
                    mkdir(GOATAddress) ;
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
                if strcmp(D.Field.Name,'uo')
                    FieldDescription = 'Topographic Mask' ;
                    try
                        save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    catch
                        mkdir(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','')) ;
                        save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    end
                end
            end
            
            for m = 1:12
                if strcmp(D.Field.LevelType,'3D')
                    data = [] ;
                    for i = 1:5
                        try
                            eval(['if length(size(Data.var',int2str(i),')) == 4 ; data = Data.var',int2str(i),'(:,:,:,m)  ; end']) ;
                        end
                        try
                            eval(['if length(size(Data.dim',int2str(i),')) == 4 ; data = Data.dim',int2str(i),'(:,:,:,m) ; end']) ;
                        end
                    end
                else
                    for i = 1:5
                        try
                            eval(['if length(size(Data.var',int2str(i),')) == 3 ; data = Data.var',int2str(i),'(:,:,m) ; end']) ;
                        end
                        try
                            eval(['if length(size(Data.dim',int2str(i),')) == 3 ; data = Data.dim',int2str(i),'(:,:,m) ; end']) ;
                        end
                    end
                end
                FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000),'.mat']);
                data(data>1.e30) = nan ;
                try
                    save(FileAddress,'data') ;
                catch
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                if strcmp(D.Field.Name,'uo')
                    bata = [] ;
                    for i = 1:5
                        try
                            eval('if length(size(Data.var',int2str(i),')) == 3 ; bata = Data.var',int2str(i),'(:,:,m) ; end') ;
                        end
                        try
                            eval('if length(size(Data.dim',int2str(i),')) == 3 ; bata = Data.dim',int2str(i),'(:,:,m) ; end') ;
                        end
                    end
                    try
                        save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(m*10000),'.mat']),'bata') ;
                    catch
                        mkdir(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),'')) ;
                        save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(m*10000),'.mat']),'bata') ;
                    end
                end
            end
            delete(fullfile(DownloadFolder,FileName)) ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
        
end

return