% GOAT produce monthly means from daily data
% Copyright (C) Ori Adam Jan.2012.
% edited Dec.2013 by Ori Adam
% syntax:
% >> GOAT_Produce_Monthly_Means_From_Daily(Dailydir,varargin)
% Input arguments:
% Dailydir: specify the 'Daily' folder, Monthly means will be created in a 'Monthly' folder in the same directory as the Daily folder
% varargin:
% 'include', {'Field1','Field2','Field3', ...}: convert only the fields in the list
% 'exclude', {'Field1','Field2','Field3', ...}: do not convert the fields in the list
% NOTE: if both 'include' and 'exclude' are used, fields appearing in both the 'include' and 'exclude' list will not be convereted.
% EXAMPLE1: GOAT_Produce_Monthly_Means_From_Daily('include','pr')
%           produce monthly means only of the field 'pr' from a Daily data folder chosen manually
% EXAMPLE2: GOAT_Produce_Monthly_Means_From_Daily('dir','Volumes/Data/NCEP/Daily','include',{'pr','ua'})
%           produce monthly means only of the fields 'pr' and 'ua' from a the Daily data folder 'Volumes/Data/NCEP/Daily'
% EXAMPLE3: GOAT_Produce_Monthly_Means_From_Daily('dir','Volumes/Data/NCEP/Daily','exclude',{'BoerBeta','wap')
%           produce monthly means of all fields except for 'BoerBeta' and 'wap' from a the Daily data folder 'Volumes/Data/NCEP/Daily'
%

function GOAT_Produce_Monthly_Means_From_Daily(Dailydir,varargin)
% Organize and distribute data using the GOAT2013 standard
% GOAT2013 standard: 'Time Resolution --> 'Field' --> 'year' --> month*10000 + Day*100 + Hour(00:1:23)
% Example: 'pr' at Nov.28.75, 12:00 sits at: 'DailyX4/pr/1975/112812.mat'
% Example: 'pr' November monthly mean sits at: 'Monthly/pr/1975/110000.mat'
% INPUT:
% if no input arguments are given, the funtion will ask for a 'Daily' directory and produce monthly means of all fields in
% that directory, savint them in a folder named 'Monthly', located in the same directory as the 'Daily' folder.
% Valid Daily Calendars are: 'Daily', 'Daily365' and 'Daily360'
% input arguments are
% 1. 'dir','Daily_Folder_Address': specify the 'Daily' folder
% 2. 'include', {'Field1','Field2','Field3', ...}: convert only the fields in the list
% 3. 'exclude', {'Field1','Field2','Field3', ...}: do not convert the fields in the list
% NOTE: if 'include' and 'exclude' are both used, only fields appearing in the 'include' list and not appearing in the 'exclude' list will be convereted.
% EXAMPLE1: GOAT_Produce_Monthly_Means('include','pr')
%           produce monthly means only of the field 'pr' from a Daily data folder chosen manually
% EXAMPLE2: GOAT_Produce_Monthly_Means('dir','Volumes/Data/NCEP/Daily','include',{'pr','ua'})
%           produce monthly means only of the fields 'pr' and 'ua' from a the Daily data folder 'Volumes/Data/NCEP/Daily'
% EXAMPLE3: GOAT_Produce_Monthly_Means('dir','Volumes/Data/NCEP/Daily','exclude',{'BoerBeta','wap')
%           produce monthly means of all fields except for 'BoerBeta' and 'wap' from a the Daily data folder 'Volumes/Data/NCEP/Daily'
%
G.EnvironmentFlag = 1  ; % Requird for GOAT_Display_message
Verbosity = 1 ; % 0 | 1 | 2 levels of verbosity
Fields2Include = {''} ;
Fields2Exclude = {''} ;
if nargin == 0
    Dailydir = uigetdir ;
end
try
Monthlydir = fullfile(fileparts(Dailydir), 'Monthly','') ;
catch
     disp 'GOAT_Produce_Monthly_Means_From_Daily exited due to invalid Daily directory.'
     return
end
if nargin > 1
    for j = 1:length(varargin)/2
        switch lower(varargin{j*2-1})
            case 'include'
                if ~iscell(varargin{j*2})
                    Fields2Include = varargin(j*2) ;
                else
                    Fields2Include = varargin{j*2} ;
                end
            case 'exclude'
                if ~iscell(varargin{j*2})
                    Fields2Exclude = varargin(j*2) ;
                else
                    Fields2Exclude = varargin{j*2} ;
                end
            otherwise
                if Verbosity
                    disp(['Unrecognized Option: ',varargin{j*2}])
                    disp 'Produce_Monthly_Means aborted'
                end
                return
        end
    end
    % else
    %     try
    %         Dailydir = uigetdir(cd,'Daily Data') ;
    %         Monthlydir = fullfile(fileparts(Dailydir), 'Monthly','') ;
    %     catch
    %         return ;
    %     end
end
if isempty(Dailydir)
    Dailydir = uigetdir(cd,'Daily Data') ;
    try
    Monthlydir = fullfile(fileparts(Dailydir), 'Monthly','') ;
    catch
     disp 'GOAT_Produce_Monthly_Means_From_Daily exited due to invalid Daily directory.'
     return
    end
end

% determine what type of Daily Calendar:
ValidCalendars = {'Daily','Daily365','Daily360'} ;
[dailyPath,DailyCalendar] = fileparts(Dailydir) ;

if ~ismember(DailyCalendar,ValidCalendars)
    if Verbosity
        ValidCalendarsStr = [] ;
        for i = 1:length(ValidCalendars)
            ValidCalendarsStr = [ValidCalendarsStr,ValidCalendars{i},' | '] ;
        end
        ValidCalendarsStr = ValidCalendarsStr(1:end-2) ;
        disp (['Invalid Daily Calendar. Valid calendars are: ',ValidCalendarsStr]) ;
    end
    return ;
end
[YYY,X] = GOAT_ParseTimeResolution(DailyCalendar) ;

tic ;

if isempty(Fields2Include{:})
    % Locate Existing Fields
    Fieldsdir = dir(Dailydir) ;
    Fields2Include = cell(1,1) ;
    NF = 0 ;
    for j = 1:length(Fieldsdir)
        if Fieldsdir(j,1).isdir && ...
                ~strcmp(Fieldsdir(j,1).name,'.') && ...
                ~strcmp(Fieldsdir(j,1).name,'..')
            NF = NF + 1 ;
            Fields2Include(NF,1) = {Fieldsdir(j,1).name} ;
        end
    end
end
GOAT_Display_message(G,'','init')
for f = 1 : length(Fields2Include)
    if sum(strcmp(Fields2Include{f},Fields2Exclude)) ; continue ; end
    yearsdir = dir(fullfile(Dailydir,Fields2Include{f},'')) ;
    NF = 0 ;
    years = [] ;
    for j = 1:length(yearsdir)
        if yearsdir(j,1).isdir && ...
                ~strcmp(yearsdir(j,1).name,'.') && ...
                ~strcmp(yearsdir(j,1).name,'..')
            NF = NF + 1 ;
            years(NF) = round(str2double(yearsdir(j,1).name)) ;
        end
    end
    % Transfer Dims Files
    if ~exist(fullfile(Monthlydir,cell2mat(Fields2Include(f)),'Dims.mat'),'file') ;
        if ~exist(fullfile(Dailydir,cell2mat(Fields2Include(f)),'Dims.mat'),'file') ;
            GOAT_Display_message(G,['Warning: Missing ''Dims.mat'' file in Daily field. --> ',fullfile(Monthlydir,cell2mat(Fields2Include(f)),'Dims.mat')],'keepthis')
        else
            try
                mkdir(fullfile(Monthlydir,cell2mat(Fields2Include(f)))) ;
                copyfile(fullfile(Dailydir,cell2mat(Fields2Include(f)),'Dims.mat'),fullfile(Monthlydir,cell2mat(Fields2Include(f)),'Dims.mat')) ;
            catch
                if Verbosity
                    GOAT_Display_message(G,['Warning: could not produce Dims file --> ',fullfile(Monthlydir,cell2mat(Fields2Include(f)),'Dims.mat')],'keepthis')
                end
            end
        end
    end
    for y = 1: length(years)
        switch Verbosity
            case 1
                GOAT_Display_message(G,['Field ',int2str(f),' of ',int2str(length(Fields2Include)),' - ',fullfile(Monthlydir,cell2mat(Fields2Include(f)),num2str(years(y)),'')],'stat')  ;
            case 2
                if y == 1
                    GOAT_Display_message(G,['Field ',int2str(f),' of ',int2str(length(Fields2Include)),' - ',fullfile(Monthlydir,cell2mat(Fields2Include(f)))],'stat') ;
                end
        end
        for m = 1 : 12
            mdays = GOAT_DaysInMonth(m,years(y),DailyCalendar) ;
            try
                if strcmp(Fields2Include(f),'BoerBeta')
                    load(fullfile(Dailydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+100),'.mat'])) ;
                    datam = zeros([mdays,size(bata)]) ;
                    for k = 1 : mdays
                        clear bata
                        load(fullfile(Dailydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+k*100),'.mat'])) ;
                        if length(size(bata)) == 3
                            datam(k,:,:,:) = bata ;
                        else
                            datam(k,:,:) = bata ;
                        end
                    end
                    bata = squeeze(single(nanmean(datam,1))) ;
                    clear datam ;
                    FileAddress = fullfile(Monthlydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000),'.mat']) ;
                    if ~exist(fileparts(FileAddress),'dir')
                        mkdir(fileparts(FileAddress)) ;
                    end
                    save(FileAddress,'bata') ;
                else
                    load(fullfile(Dailydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+100),'.mat'])) ;
                    datam = zeros([mdays,size(data)]) ;
                    for k = 1 : mdays
                        clear data
                        load(fullfile(Dailydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000+k*100),'.mat'])) ;
                        if length(size(data)) == 3
                            datam(k,:,:,:) = data ;
                        else
                            datam(k,:,:) = data ;
                        end
                    end
                    data = squeeze(single(nanmean(datam,1))) ;
                    clear datam ;
                    FileAddress = fullfile(Monthlydir,cell2mat(Fields2Include(f)),num2str(years(y)),[int2str(m*10000),'.mat']) ;
                    if ~exist(fileparts(FileAddress),'dir')
                        mkdir(fileparts(FileAddress)) ;
                    end
                    save(FileAddress,'data') ;
                end
            catch
                % Nevermind, Missing days
            end
        end
    end
    
end
t = toc ;
[h,m,s,TimeString] = GOAT_UTILS_seconds2time(t) ;

% if h > 0
%     TimeString = [int2str(h),' hours, ',int2str(m),' minutes and ',int2str(s),' seconds.'] ;
% elseif m > 0
%     TimeString = [int2str(m),' minutes and ',int2str(s),' seconds.'] ;
% elseif s > 0
%     TimeString = [int2str(s),' seconds.'] ;
% end
if Verbosity
    disp(['Daily to Monthly conversion completed in ',TimeString]) ;
end