% GOAT_Analysis_Tropopause_Height.  Ori Adam. July.2013.
% Calculates Tropopause height (pressure) in isobaric coordinates Based on the method described in Reichler et al. 2003.
% the tropopause height is defined as (WMO):
% First level at which the lapse rate <= 2K/km and for which no level exists for 2km above, in which the lapse rate > 2K/km
% Syntax:
% Pt = GOAT_Analysis_Tropopause_Height(T,p,G)
% Arguments: 
% T = temperature field (latitude,pressure) or (1,latitude,pressure)
% p = pressure [mb] vector or (latitude,pressure) matrix
% G = GOAT structure (optional)
% Pt = tropopause pressure with length = first non-singleton dimension
%

function Pt = GOAT_Analysis_Tropopause_Height(T,p,G)

if nargin > 2
    P = double(p) ;
    Factor = G.Consts.GravitationalConst/G.Consts.SpHeatConstPr*1000 ;
    k = G.Consts.IdealGasRdry/G.Consts.SpHeatConstPr ;
else
    P = double(G.P) ;
    Factor = 9.8062/1004.6*1000 ;
    k = 287/1004.6 ;
end
%

PI = (fliplr(1:1000)*100).^k ;

if length(size(T)) == 2
    T = reshape(T,1,size(T,1),size(T,2)) ;
end

if min(size(p)) > 1
    if P(1,end) >  P(1,1)
    P = flipdim(P,2)    ;
    T = flipdim(T,3) ;
    end
    Pk = repmat(reshape(P*100.^k,[1 size(P)]),[size(T,1) 1 1]) ;
else
if P(end) >  P(1)
    [pm Ip] = max(size(P)) ;
    P = flipdim(P,Ip)    ;
    T = flipdim(T,3) ;
end    
Pk = repmat(reshape((P*100).^k,1,1,length(P)),[size(T,1) size(T,2) 1]) ;
end
Pk2 = (Pk(:,:,1:end-1) + Pk(:,:,2:end))/2 ;
%
T2  = (T(:,:,1:end-1) + T(:,:,2:end))/2 ;
Pk1 = Pk2(1,1,:) ;

Gamma = reshape((T(:,:,2:end)-T(:,:,1:end-1))./(Pk(:,:,2:end)-Pk(:,:,1:end-1)).*Pk2./T2.*Factor,size(T,1)*size(T,2),size(T,3)-1) ; % Lapse rate in K/km
T2    = reshape(T2,size(T,1)*size(T,2),size(T,3)-1) ;
%
Pt = zeros(size(T,1)*size(T,2),1) ;
for j = 1:size(Gamma,1)
    G1 = interp1(Pk1,double(Gamma(j,:)'),PI,'linear','extrap') ;
    T1 = interp1(Pk1,double(T2(j,:)'),PI,'linear','extrap') ;
    idx = find(G1 <=2 & PI < (550*100).^k & PI > (75*100).^k) ;
    Pidx = PI(idx) ;
    if isempty(Pidx)
        Pt(j) = nan ;
    else
        for c = 1:length(Pidx)
            dpk_2km = -2000*k*G.Consts.GravitationalConst/G.Consts.IdealGasRdry./T1(c).*Pidx(c) ;
            idx2 = dsearchn(Pidx(c:end)',Pidx(c) + dpk_2km) ;
            if sum(G1(idx(c):idx(c)+idx2-1) <=2) == idx2 ; % Applying the condition that at no level in the above 2km the lapse rate >2K/km 
                Pt(j) = Pidx(c) ;
                break ;
            else
                continue ;
            end
        end
    end
end

Pt = reshape(Pt,size(T,1),size(T,2)) ; 
try
Pt = inpaintn(Pt) ; % interpolate over nan values if inpaint is available
end
Pt = Pt.^(1/k)/100 ;


%figure ; plot(lat,mean(Pt,1)) ; axis ij ;

