function L = GOAT_Calculate_L(T,G, type)
% SATURATION_THERMODYNAMICS  Saturation quantities for water. Adapted from Tapio S. Code
%
% returns latent heat of vaporization/sublimation. Depending on
% type, the function uses different formulations:
%    'era': Formulation used by ECMWF based on the modified
%           Tetens formula.
%
% 'simple': Formulation used in idealized GCM experiments by Paul
%           O'Gorman and others with fixed latent heat.
%


if nargin < 3
    type = 'simple';
end


switch lower(type)
    
    case 'era'
        
        % ECMWF formulation described by Simmons et al. (1999: QJRMS, 125,
        % 353--386), which uses saturation over ice for temperatures
        % less than 250 K and a quadratic interpolation between
        % saturation over ice and over liquid water for temperatures
        % between 250 K and 273 K
        % coefficients in Tetens saturation vapor pressure es = es0 * exp(a3 * (T-T0)/(T-a4))
        T0        = 273.16;  % (K)
        Ti        = T0 - 23; % (K)
        % latent heat of sublimation
        Ls0       = 2.834e6;  % latent heat of sublimation  (J / kg) [+- 0.01 error for 173 K < T < 273 K]
        Ls        = Ls0 * ones(size(T));
        % latent heat of vaporization
        Lv0       = 2.501e6;  % latent heat of vaporization at triple point (J / kg)
        cpl       = 4190;     % heat capacity of liquid water (J / kg / K)
        cpv       = 1870;     % heat capacity of water vapor (J / kg / K)
        Lv        = Lv0 - (cpl - cpv) .* (T - T0);
        % compute saturation vapor pressure and latent heat over liquid/ice mixture
        iice      = find(T <= Ti);
        iliquid   = find(T >= T0);
        imixed    = find(T > Ti & T < T0);
        L         = zeros(size(T));
        if ~isempty(iice)
            L(iice)   = Ls(iice);
        end
        if ~isempty(iliquid)
            L(iliquid)  = Lv(iliquid);
        end
        if ~isempty(imixed)
            a         = ( (T(imixed) - Ti)./(T0 - Ti) ).^2;
            L(imixed) = (1-a) .* Ls(imixed) + a .* Lv(imixed);
        end
    otherwise
        L         = G.Consts.LatentHeatOfVapor ;
end
