% GOAT Customized Fiels Display Settings
% Copyright (C) Ori Adam Sep.2013. Compatible with version 2013i or later.
% This function accepts the Field Name (IPCC standard) and sets the default display settings for this field
% Example 1: pr, precipitation rate will be set to
% a) symmetric Z Limits
% b) blue white red countourf colormap
% c) atmospheric topography
% d) line width  = 'none' in countourF
% e) Coast LineWidth = 2 ;
% Example 2: zonal wind Lat-Lon projection
% a) same as pr
% for Lat-Haeight Projection
% a) symmetric Z Limits
% b) blue white red countourf colormap
% c) atmospheric topography
% d) line width  = '1' in countourF
% Example 3: SST
% a) for lon-lat projection, bluewhitered colormap
% b) for lat-height projection (implies zonal averaging), set the NanMean flag to on
% Some repeating Custom Displays are set in GOAT_Defaults
%
% see also GOAT_custom_field_settings, GOAT_VirtualField_, GOAT_VirtualFields, GOAT_plotUpdate, GOAT_Defaults


function G = GOAT_FieldSettings(G)
try
    DataField = G.ALP.Defs.(G.DataField).ReferenceSettingsField ; % Linked field with Defs.ReferenceSettingsField
catch
    G.VirtualFields.ReferenceFieldSettings  = [] ;
    switch G.DataField
        case G.VirtualFields.List
            G = GOAT_VirtualFields(G,'ReferenceField',G.DataField) ;
            if ~isempty(G.VirtualFields.ReferenceFieldSettings)
                DataField = G.VirtualFields.ReferenceFieldSettings ;
            else
                DataField = G.DataField ;
            end
        case G.VirtualFields.MyList
            Idx = find(strcmp(G.DataField,G.VirtualFields.MyList)) ;
            eval(['G = ',G.VirtualFields.MyFuncs{Idx},'(''ReferenceField'',G) ; ']) ;
            if ~isempty(G.VirtualFields.ReferenceFieldSettings)
                DataField = G.VirtualFields.ReferenceFieldSettings ;
            else
                DataField = G.DataField ;
            end
        otherwise
            DataField = G.DataField ;
    end    
end
switch DataField
    case {'pr','er'}
        if ~G.Settings.NanMeanFlag
            G.Settings.NanMeanFlag       = G.Settings.Custom1.NanMeanFlag       ; % use mean(x) function when 0 and nanmean(x) when 1 ;
        end
        G.Settings.ContourIdx        = G.Settings.Custom1.ContourIdx        ; % Contour Type Idx
        G.Settings.WaitbarCutoff     = G.Settings.Custom1.WaitbarCutoff     ; % Number of data loadings above which the waitbar appears
        G.Settings.ColorMapIdx       = G.Settings.Custom1.ColorMapIdx       ; % Default ColorMap Index
        G.Settings.CoastLineWidthIdx = G.Settings.Custom1.CoastLineWidthIdx ; % Default LineWidth Index in Plots
        G.Settings.XgridFlag         = G.Settings.Custom1.XgridFlag         ; % A flag for displaying the X Grid Lines
        G.Settings.YgridFlag         = G.Settings.Custom1.YgridFlag         ; % A flag for displaying the Y Grid Lines
        G.Settings.SymZLimitsFlag    = G.Settings.Custom1.SymZLimitsFlag                                    ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
        G.Settings.FixZLimitsFlag    = G.Settings.Custom1.FixZLimitsFlag    ; % A flag for fixing limits (useful in auto time steps)
        G.Settings.LogLevelsFlag     = G.Settings.Custom1.LogLevelsFlag     ; % log(P) in lat-P,Lon-P projections
        G.Settings.ContourLevelStep  = G.Settings.Custom1.ContourLevelStep  ; % contour step. 0 = auto
        G.Settings.ReverseColorMap   = G.Settings.Custom1.ReverseColorMap   ; % Reverse Colormap
        G.topo.TypeFlag              = 2                                    ; % 1) None 2) Atmo 3) Ocean 4) All
        switch G.ProjectionType
            case {3,4}
                G.Settings.LineWidthIdx  = 1                                    ; % Default LineWidth Index in Plots
                G.Settings.SymZLimitsFlag = 0 ;
            otherwise
                G.Settings.LineWidthIdx  = G.Settings.Custom1.LineWidthIdx      ; % Default LineWidth Index in Plots
        end
        G.Settings.LineWidth         = (G.Settings.LineWidthIdx-2)/2 ;
        G.Settings.CoastLineWidth    = G.Settings.Custom1.CoastLineWidthIdx/2 - .5 ;
        if sum(strcmp(DataField,{'pr','er'}) )
            G.Settings.ReverseColorMap   = 0 ;
            G.Settings.ColorMapIdx      = find(strcmp(G.Settings.ColorMap,'Blues_seq_cbrewer'))  ; % BlueWhiteRed
            G.Settings.SymZLimitsFlag = 0 ;
        end
    case {'uas','vas','ua','va','Psi','wap','PsiH','PsiL'}
        if ~G.Settings.NanMeanFlag
            G.Settings.NanMeanFlag       = G.Settings.Custom2.NanMeanFlag       ; % use mean(x) function when 0 and nanmean(x) when 1 ;
        end
        G.Settings.ContourIdx        = G.Settings.Custom2.ContourIdx        ; % Contour Type Idx
        G.Settings.WaitbarCutoff     = G.Settings.Custom2.WaitbarCutoff     ; % Number of data loadings above which the waitbar appears
        G.Settings.ColorMapIdx       = G.Settings.Custom2.ColorMapIdx       ; % Default ColorMap Index
        G.Settings.CoastLineWidthIdx = G.Settings.Custom2.CoastLineWidthIdx ; % Default LineWidth Index in Plots
        G.Settings.XgridFlag         = G.Settings.Custom2.XgridFlag         ; % A flag for displaying the X Grid Lines
        G.Settings.YgridFlag         = G.Settings.Custom2.YgridFlag         ; % A flag for displaying the Y Grid Lines
        G.Settings.SymZLimitsFlag    = G.Settings.Custom2.SymZLimitsFlag                                    ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
        G.Settings.FixZLimitsFlag    = G.Settings.Custom2.FixZLimitsFlag    ; % A flag for fixing limits (useful in auto time steps)
        G.Settings.LogLevelsFlag     = G.Settings.Custom2.LogLevelsFlag     ; % log(P) in lat-P,Lon-P projections
        G.Settings.ContourLevelStep  = G.Settings.Custom2.ContourLevelStep  ; % contour step. 0 = auto
        G.Settings.ReverseColorMap   = G.Settings.Custom2.ReverseColorMap   ; % Reverse Colormap
        G.topo.TypeFlag              = 2                                    ; % 1) None 2) Atmo 3) Ocean 4) All
        switch G.ProjectionType
            case {3,4}
                G.Settings.LineWidthIdx  = 1                                    ; % Default LineWidth Index in Plots
            otherwise
                G.Settings.LineWidthIdx  = G.Settings.Custom2.LineWidthIdx      ; % Default LineWidth Index in Plots
        end
        G.Settings.LineWidth         = (G.Settings.LineWidthIdx-2)/2 ;
        G.Settings.CoastLineWidth    = G.Settings.Custom2.CoastLineWidthIdx/2 -.5 ;
    case {'ta','tas','to','tos','hus','hur','zg'} % always much larger than zero
        if ~G.Settings.NanMeanFlag
            G.Settings.NanMeanFlag       = G.Settings.Custom3.NanMeanFlag       ; % use mean(x) function when 0 and nanmean(x) when 1 ;
        end
        G.Settings.ContourIdx        = G.Settings.Custom3.ContourIdx        ; % Contour Type Idx
        G.Settings.WaitbarCutoff     = G.Settings.Custom3.WaitbarCutoff     ; % Number of data loadings above which the waitbar appears
        G.Settings.ColorMapIdx       = G.Settings.Custom3.ColorMapIdx       ; % Default ColorMap Index
        G.Settings.CoastLineWidthIdx = G.Settings.Custom3.CoastLineWidthIdx ; % Default LineWidth Index in Plots
        G.Settings.XgridFlag         = G.Settings.Custom3.XgridFlag         ; % A flag for displaying the X Grid Lines
        G.Settings.YgridFlag         = G.Settings.Custom3.YgridFlag         ; % A flag for displaying the Y Grid Lines
        G.Settings.SymZLimitsFlag    = G.Settings.Custom3.SymZLimitsFlag                                    ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
        G.Settings.FixZLimitsFlag    = G.Settings.Custom3.FixZLimitsFlag    ; % A flag for fixing limits (useful in auto time steps)
        G.Settings.LogLevelsFlag     = G.Settings.Custom3.LogLevelsFlag     ; % log(P) in lat-P,Lon-P projections
        G.Settings.ContourLevelStep  = G.Settings.Custom3.ContourLevelStep  ; % contour step. 0 = auto
        G.Settings.ReverseColorMap   = G.Settings.Custom3.ReverseColorMap   ; % Reverse Colormap
        G.topo.TypeFlag              = 2                                    ; % 1) None 2) Atmo 3) Ocean 4) All
        switch G.ProjectionType
            case {3,4}
                G.Settings.LineWidthIdx  = 1                                    ; % Default LineWidth Index in Plots
            otherwise
                G.Settings.LineWidthIdx  = G.Settings.Custom3.LineWidthIdx      ; % Default LineWidth Index in Plots
        end
        G.Settings.LineWidth         = (G.Settings.LineWidthIdx-2)/2 ;
        G.Settings.CoastLineWidth    = G.Settings.Custom3.CoastLineWidthIdx/2 -.5;
    otherwise
        if ~G.Settings.NanMeanFlag
            G.Settings.NanMeanFlag       = G.Settings.Default.NanMeanFlag       ; % use mean(x) function when 0 and nanmean(x) when 1 ;
        end
        G.Settings.ContourIdx        = G.Settings.Default.ContourIdx        ; % Contour Type Idx
        G.Settings.WaitbarCutoff     = G.Settings.Default.WaitbarCutoff     ; % Number of data loadings above which the waitbar appears
        G.Settings.ColorMapIdx       = G.Settings.Default.ColorMapIdx       ; % Default ColorMap Index
        G.Settings.ReverseColorMap   = G.Settings.Default.ReverseColorMap   ; % Reverse Colormap
        G.Settings.CoastLineWidthIdx = G.Settings.Default.CoastLineWidthIdx ; % Default LineWidth Index in Plots
        G.Settings.CoastLineWidth    = G.Settings.Default.CoastLineWidth    ;
        G.Settings.XgridFlag         = G.Settings.Default.XgridFlag         ; % A flag for displaying the X Grid Lines
        G.Settings.YgridFlag         = G.Settings.Default.YgridFlag         ; % A flag for displaying the Y Grid Lines
        G.Settings.SymZLimitsFlag    = G.Settings.Default.SymZLimitsFlag    ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
        G.Settings.FixZLimitsFlag    = G.Settings.Default.FixZLimitsFlag    ; % A flag for fixing limits (useful in auto time steps)
        G.Settings.LogLevelsFlag     = G.Settings.Default.LogLevelsFlag     ; % log(P) in lat-P,Lon-P projections
        G.topo.TypeFlag              = G.Settings.Default.topoTypeFlag      ; % 1) None 2) Atmo 3) Ocean 4) All
        switch G.ProjectionType
            case {3,4}
                G.Settings.LineWidthIdx  = 1                                    ; % Default LineWidth Index in Plots
            otherwise
                G.Settings.LineWidthIdx  = G.Settings.Default.LineWidthIdx      ; % Default LineWidth Index in Plots
        end
        G.Settings.LineWidth         = (G.Settings.LineWidthIdx-2)/2 ;
end
if any(G.ProjectionType == [1,2,8,9])
    switch G.topo.TypeFlag
        case {1,2}
            G.topo.intervalmin = 0     ;
            G.topo.intervalmax = 8000  ;
        case 3
            G.topo.intervalmin = -8000 ;
            G.topo.intervalmax =  0    ;
        case 4
            G.topo.intervalmin = -8000 ;
            G.topo.intervalmax = 8000  ;
    end
    G.topo.topovec = G.topo.intervalmin:G.topo.interval:G.topo.intervalmax ;
    try
        zeroidx = dsearchn(G.topo.topovec',0) ;
        G.topo.topovec = G.topo.topovec(G.topo.topovec~=G.topo.topovec(zeroidx))  ;
    catch
        G.topo.topovec = 0 ;
    end
    G.topo.TopotLineWidthIdx = 2 ; % '0.5 | 1 | 1.5 | 2 | 2.5 | 3 | 3.5 | 4 | 4.5 | 5 | 5.5 | 6 | 6.5 | 7 | 7.5 | 8 | 8.5 | 9 | 9.5 | 10 '
end

% Customized Settings
G = GOAT_MySets(G,G.DataField,'Field') ; 
if G.EnvironmentFlag == 1
    G.Settings.WaitbarCutoff = 1.e11 ;
else
G = update_rhs_settings(G) ;
end
% update RHS settings
function G = update_rhs_settings(G)
try
    %set(G.rhs_Contour       ,'Value',G.Settings.ContourIdx       ) ; % right hand settings Contour Type Idx
    set(G.rhs_ColorMap      ,'Value',G.Settings.ColorMapIdx      ) ; % color map index
    set(G.rhs_CoastLineWidth,'Value',G.Settings.CoastLineWidthIdx) ; % Coastline width index
    set(G.rhs_Xgrid         ,'Value',G.Settings.XgridFlag        ) ; % X-Gridlines
    set(G.rhs_Ygrid         ,'Value',G.Settings.YgridFlag        ) ; % Y-Gridlines
    set(G.rhs_SymZ          ,'Value',G.Settings.SymZLimitsFlag   ) ; % symmetric Z limits
    set(G.rhs_FixZ          ,'Value',G.Settings.FixZLimitsFlag   ) ; % fix Z-limits (Climits)
    set(G.rhs_SinLat        ,'Value',G.Settings.SinLatFlag       ) ; % plot sin(latitude)
    set(G.rhs_LogLevels     ,'Value',G.Settings.LogLevelsFlag    ) ; % log-Y levels
    set(G.rhs_LineWidth     ,'Value',G.Settings.LineWidthIdx     ) ; % contour/plot linewidth
    set(G.rhs_StepLevel    ,'string',num2str(G.Settings.ContourLevelStep,5)) ; % contour step level. 0 = auto
    set(G.rhs_RevColMap     ,'Value',G.Settings.ReverseColorMap) ; % Reverse Color Map Flag
    uicontrol(G.h_DataSelectionT) ; % move focus back to fig to allow keypress
catch
    % nevermind
end