% Produce 3D mean of a GOAT Field
% INPUT: - Data Folder Address
%        - Field Name
%        - temporal span
%        - Climatology Flag
% OUTPUT: 3D mean/climatology, ignoring nan values    % Note: Filters and weights are disabled!
%
% It is recommended though not mandatory to work from the ... GOAT/Code directory                                                          
% When not working from the GOAT/Code directory make sure that the following folders                                                       
% with all subfolders are added to the matlab path:                                                                                        
% ... GOAT/Code, GOAT/Scripts,   and GOAT/MyScripts                                                                                      
% cell-1: Initiate GOAT  and set Dataset, Field and temporal resolution                                                                    
% cell-2: set temporal limits                                                                                                              
% cell-3: set spatial limits and Filters/weights                                                                                           
% cell-4: retrieve and compile 3D data                                                                                                                                                                                                                                                                                                                                                                                                                                                      
                                                                                                                                                          
%% cell-1: Initiate GOAT and set dataset, field and time resolution                                                                                       
G = GOAT_Environment('enter_project_path') ; % Initiate GOAT Environment                                                                                  
G = GOAT_Initialize_Dataset(G,'GOAT/Data/ERA-Interim','Monthly') ; % Go through contents of Data folder. identify existing fields, temporal span, etc.
G = GOAT_Initialize_Field(G,'ua')        ; % get spatial and temporal dimensions and display settings                                                      
%% cell-2: Temporal limits                                                                                                                                
% Temporal Checkboxes/Flags                                                                                                                               
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1                                                                                         
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag                                                                             
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots                                                      
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots                                                                                  
year1  = 1979 ; % yyyy                                                                                                                                    
month1 = 1    ; % 1:12                                                                                                                                    
day1   = 1    ; % 1:31                                                                                                                                    
hour1  = 0    ; % 0:3:21                                                                                                                                  
year2  = 1983 ;                                                                                                                                           
month2 = 12    ;                                                                                                                                          
day2   = 1    ;                                                                                                                                           
hour2  = 0    ;                                                                                                                                           
% G.Limits.S1 and G.Limits.S1 are the temporal indices used by GOAT to locate files in the GOAT archiving standard                                        
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ;                                                                           
if G.Limits.Default.Day2same && G.Limits.Default.Sequential                                                                                               
    G.Limits.S2 = G.Limits.S1 ;                                                                                                                           
else                                                                                                                                                      
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ;                                                                           
end                                                                                                                                                       
%% cell-3: Spatial limits                                                                                                                                 
% Checkboxes/Flags                                                                                                                                        
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height                
G.Limits.Default.sameP = 1      ; % Projection 1,2) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels                                      
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate                       
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1                                                                                  
Level1  = 300   ; % GOAT will use the nearest level to this value                                                                                           
Level2  = 1000  ; % GOAT will use the nearest level to this value                                                                                          
Lon1    = 0     ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified                                                     
Lon2    = 358.5 ; % use values from -360 to 360. if Lon2 < Lon1 the interval Lon2 < lon < Lon1 will not be used.                                        
Lat1    = 90    ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.                                                     
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.                                                                                            
G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ;                                                                                                   
G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ;                                                                                                   
G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ;                                                                                                           
G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ;                                                                                                           
G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ;                                                                                                           
G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ;                                                                                                           
% Filters, Weights and Indices:                                                                                                                           
G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index                                                                                    
%% cell-4 calculate 3D mean of time series
G.DataStringVec = GOAT_Data_string_vec(G)  ; % A vecotor of the Time Series File Indices
% G.Addresses.Default(G.DataStringVec) is a vector of addresses of the files to be retreived
% use zeroed first file of time series as a default zeros matrix 
Z = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(1))),1)*0 ; 
DataMean    = Z ;  
DataCounter = Z ;  
NF  = length(G.DataStringVec) ;
Nmod = ceil(NF/10) ;
% Spatial Weights
G.Filter.weight = WeightFunc(G) ;
for i = 1:NF
            DataNans    = Z+1 ; % 
            [data,G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i))),i) ; % Load the data File
            DataNans(isnan(data)) = 0 ; 
            data(isnan(data))     = 0 ; 
            if mod(i,Nmod) == 0
            disp([' . . . Processing Composite Data . . .  ',num2str(i/NF*100,'%02.0f'),' %']) ;
            end
            DataMean = DataMean + data ;
            DataCounter = DataCounter + DataNans ; 
end
disp 'Done !' ;
DataMean = DataMean./DataCounter ; 
